@@ -1,5 +1,5 @@
 # flake8: noqa
-__version__ = '1.2.2'
+__version__ = '1.4.1'
 
 # Glue the package together
 # See https://www.youtube.com/watch?v=0oTh1CXRaQ0 if this confuses you
@@ -11,11 +11,13 @@ from strax.processing.general import *
 
 from .storage.common import *
 from .storage.files import *
+from .storage.file_rechunker import *
 from .storage.mongo import *
 from .storage.zipfiles import *
 
 from .config import *
-from .plugin import *
+from .plugins import *
+
 from .mailbox import *
 from .processor import *
 from .context import *
