@@ -4,26 +4,51 @@ import strax
 import numba
 from numba.typed import List
 import numpy as np
+import warnings
 
 export, __all__ = strax.exporter()
 
 
-# (5-10x) faster than np.sort(order=...), as np.sort looks at all fields
-# TODO: maybe this should be a factory?
 @export
-@numba.jit(nopython=True, nogil=True, cache=True)
 def sort_by_time(x):
-    """Sort pulses by time, then channel.
-
-    Assumes you have no more than 10k channels, and records don't span
-    more than 100 days. TODO: FIX this
+    """Sorts things. Either by time or by time, then channel if both
+    fields are in the given array.
     """
     if len(x) == 0:
-        # Nothing to do, and .min() on empty array doesn't work, so:
         return x
+    
+    if 'channel' in x.dtype.names:
+        min_channel = x['channel'].min()
+        channel = x['channel']
+        if min_channel < 0:
+            channel -= min_channel
+    else:
+        channel = np.ones(len(x))
+        
+    max_time_difference = (np.iinfo(np.int64).max - 10) / (channel.max()+1)
+    # Subtract 10 to have some extra margin, just in case.
+    # Use absolute to account for peaks which are channel -1.
+    _time_range_too_large = (x['time'].max() - x['time'].min()) > max_time_difference
+    if not _time_range_too_large:
+        # Faster sorting:
+        x = _sort_by_time_and_channel(x, channel, channel.max()+1)
+    elif 'channel' in x.dtype.names:
+        x = np.sort(x, order=('time', 'channel'))
+    else:
+        x = np.sort(x, order=('time',))
+    return x
+
+@numba.jit(nopython=True, nogil=True, cache=True)
+def _sort_by_time_and_channel(x, channel, max_channel_plus_one):
+    """
+    Assumes you have no more than 10k channels, and records don't span
+    more than 11 days.
+
+    (5-10x) faster than np.sort(order=...), as np.sort looks at all fields
+    """
     # I couldn't get fast argsort on multiple keys to work in numba
     # So, let's make a single key...
-    sort_key = (x['time'] - x['time'].min()) * 10000 + x['channel']
+    sort_key = (x['time'] - x['time'].min()) * max_channel_plus_one + channel
     sort_i = np.argsort(sort_key)
     return x[sort_i]
 
@@ -266,6 +291,32 @@ def overlap_indices(a1, n_a, b1, n_b):
     return (a_start, a_end), (b_start, b_end)
 
 
+@export
+def split_touching_windows(things, containers, window=0):
+    """
+    Split things by their containers and return a list of length containers
+    :param things: Sorted array of interval-like data
+    :param containers: Sorted array of interval-like data
+    :param window: threshold distance for touching check
+    For example:
+       - window = 0: things must overlap one sample
+       - window = -1: things can start right after container ends
+         (i.e. container endtime equals the thing starttime, since strax
+          endtimes are exclusive)
+    :return:
+    """
+    windows = touching_windows(things, containers, window)
+    return _split_by_window(things, windows)
+
+
+@numba.njit
+def _split_by_window(r, windows):
+    result = []
+    for w in windows:
+        result.append(r[w[0]:w[1]])
+    return result
+
+
 @export
 def touching_windows(things, containers, window=0):
     """Return array of (start, exclusive end) indices into things which extend
