@@ -21,6 +21,8 @@ class ZipDirectory(strax.StorageFrontend):
     instead these zip files are made by zipping stuff from FileSytemBackend.
     """
 
+    storage_typ = strax.StorageType.COMPRESSED
+
     def __init__(self, path='.', *args, readonly=True, **kwargs):
         if not readonly:
             raise NotImplementedError("Zipfiles are currently read-only")
@@ -75,7 +77,7 @@ class ZipDirectory(strax.StorageFrontend):
     def _zipname(self, key):
         zipname = osp.join(self.path, key.run_id + '.zip')
         # Since we're never writing, this check can be here
-        # TODO: sounds like a bad idea?
+        # is this a bad idea?
         if not osp.exists(zipname):
             raise strax.DataNotAvailable
         return zipname
@@ -109,7 +111,7 @@ class ZipFileBackend(strax.StorageBackend):
             with zp.open(dirn + '/' + chunk_info['filename']) as f:
                 return strax.load_file(f, dtype=dtype, compressor=compressor)
 
-    def get_metadata(self, zipn_and_dirn):
+    def _get_metadata(self, zipn_and_dirn):
         zipn, dirn = zipn_and_dirn
         with zipfile.ZipFile(zipn) as zp:
             prefix = strax.dirname_to_prefix(dirn)
